@@ -89,33 +89,6 @@
 
 #endif /* !NATIVE_WIN32 */
 
-#ifdef _MSC_VER
-/* Make MSVC more pedantic, this is a recommended pragma list
- * from _Win32_Programming_ by Rector and Newcomer.
- */
-#pragma warning(error:4002)
-#pragma warning(error:4003)
-#pragma warning(1:4010)
-#pragma warning(error:4013)
-#pragma warning(1:4016)
-#pragma warning(error:4020)
-#pragma warning(error:4021)
-#pragma warning(error:4027)
-#pragma warning(error:4029)
-#pragma warning(error:4033)
-#pragma warning(error:4035)
-#pragma warning(error:4045)
-#pragma warning(error:4047)
-#pragma warning(error:4049)
-#pragma warning(error:4053)
-#pragma warning(error:4071)
-#pragma warning(disable:4101)
-#pragma warning(error:4150)
-
-#pragma warning(disable:4244)	/* No possible loss of data warnings, please */
-#endif /* _MSC_VER */
-
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -182,7 +155,7 @@ extern "C" {
  * we define G_CAN_INLINE, if the compiler seems to be actually
  * *capable* to do function inlining, in which case inline function bodys
  * do make sense. we also define G_INLINE_FUNC to properly export the
- * function prototypes if no inlinig can be performed.
+ * function prototypes if no inlining can be performed.
  * we special case most of the stuff, so inline functions can have a normal
  * implementation by defining G_INLINE_FUNC to extern and G_CAN_INLINE to 1.
  */
@@ -680,7 +653,7 @@ typedef gint32	GTime;
 #  ifdef GLIB_COMPILATION
 #    define GUTILS_C_VAR __declspec(dllexport)
 #  else /* !GLIB_COMPILATION */
-#    define GUTILS_C_VAR __declspec(dllimport)
+#    define GUTILS_C_VAR extern __declspec(dllimport)
 #  endif /* !GLIB_COMPILATION */
 #else /* !NATIVE_WIN32 */
 #  define GUTILS_C_VAR extern
@@ -695,6 +668,7 @@ GUTILS_C_VAR const guint glib_binary_age;
 /* Forward declarations of glib types.
  */
 
+typedef struct _GAllocator	GAllocator;
 typedef struct _GArray		GArray;
 typedef struct _GByteArray	GByteArray;
 typedef struct _GCache		GCache;
@@ -705,7 +679,6 @@ typedef struct _GHashTable	GHashTable;
 typedef struct _GHook		GHook;
 typedef struct _GHookList	GHookList;
 typedef struct _GList		GList;
-typedef struct _GListAllocator	GListAllocator;
 typedef struct _GMemChunk	GMemChunk;
 typedef struct _GNode		GNode;
 typedef struct _GPtrArray	GPtrArray;
@@ -779,6 +752,7 @@ typedef void		(*GDataForeachFunc)	(GQuark		key_id,
 typedef void		(*GFunc)		(gpointer	data,
 						 gpointer	user_data);
 typedef guint		(*GHashFunc)		(gconstpointer	key);
+typedef void		(*GFreeFunc)		(gpointer	data);
 typedef void		(*GHFunc)		(gpointer	key,
 						 gpointer	value,
 						 gpointer	user_data);
@@ -791,6 +765,8 @@ typedef gboolean	(*GHookFindFunc)	(GHook		*hook,
 						 gpointer	 data);
 typedef void		(*GHookMarshaller)	(GHook		*hook,
 						 gpointer	 data);
+typedef gboolean	(*GHookCheckMarshaller)	(GHook		*hook,
+						 gpointer	 data);
 typedef void		(*GHookFunc)		(gpointer	 data);
 typedef gboolean	(*GHookCheckFunc)	(gpointer	 data);
 typedef void		(*GHookFreeFunc)	(GHookList      *hook_list,
@@ -865,6 +841,8 @@ struct _GDebugKey
 
 /* Doubly linked lists
  */
+void   g_list_push_allocator    (GAllocator     *allocator);
+void   g_list_pop_allocator     (void);
 GList* g_list_alloc		(void);
 void   g_list_free		(GList		*list);
 void   g_list_free_1		(GList		*list);
@@ -885,6 +863,7 @@ GList* g_list_remove		(GList		*list,
 GList* g_list_remove_link	(GList		*list,
 				 GList		*llink);
 GList* g_list_reverse		(GList		*list);
+GList* g_list_copy		(GList		*list);
 GList* g_list_nth		(GList		*list,
 				 guint		 n);
 GList* g_list_find		(GList		*list,
@@ -912,6 +891,8 @@ gpointer g_list_nth_data	(GList		*list,
 
 /* Singly linked lists
  */
+void    g_slist_push_allocator  (GAllocator     *allocator);
+void    g_slist_pop_allocator   (void);
 GSList* g_slist_alloc		(void);
 void	g_slist_free		(GSList		*list);
 void	g_slist_free_1		(GSList		*list);
@@ -932,6 +913,7 @@ GSList* g_slist_remove		(GSList		*list,
 GSList* g_slist_remove_link	(GSList		*list,
 				 GSList		*llink);
 GSList* g_slist_reverse		(GSList		*list);
+GSList*	g_slist_copy		(GSList		*list);
 GSList* g_slist_nth		(GSList		*list,
 				 guint		 n);
 GSList* g_slist_find		(GSList		*list,
@@ -955,14 +937,6 @@ gpointer g_slist_nth_data	(GSList		*list,
 #define g_slist_next(slist)	((slist) ? (((GSList *)(slist))->next) : NULL)
 
 
-/* List Allocators
- */
-GListAllocator* g_list_allocator_new  (void);
-void		g_list_allocator_free (GListAllocator* allocator);
-GListAllocator* g_slist_set_allocator (GListAllocator* allocator);
-GListAllocator* g_list_set_allocator  (GListAllocator* allocator);
-
-
 /* Hash tables
  */
 GHashTable* g_hash_table_new		(GHashFunc	 hash_func,
@@ -1051,6 +1025,8 @@ struct _GNode
 				 ((GNode*) (node))->next == NULL)
 #define	 G_NODE_IS_LEAF(node)	(((GNode*) (node))->children == NULL)
 
+void     g_node_push_allocator  (GAllocator       *allocator);
+void     g_node_pop_allocator   (void);
 GNode*	 g_node_new		(gpointer	   data);
 void	 g_node_destroy		(GNode		  *root);
 void	 g_node_unlink		(GNode		  *node);
@@ -1241,6 +1217,10 @@ void	 g_hook_list_marshal		(GHookList		*hook_list,
 					 gboolean		 may_recurse,
 					 GHookMarshaller	 marshaller,
 					 gpointer		 data);
+void	 g_hook_list_marshal_check	(GHookList		*hook_list,
+					 gboolean		 may_recurse,
+					 GHookCheckMarshaller	 marshaller,
+					 gpointer		 data);
 
 
 /* Fatal error handlers.
@@ -1361,6 +1341,16 @@ void	 g_free	       (gpointer  mem);
 void	 g_mem_profile (void);
 void	 g_mem_check   (gpointer  mem);
 
+/* Generic allocators
+ */
+GAllocator* g_allocator_new   (const gchar  *name,
+			       guint         n_preallocs);
+void        g_allocator_free  (GAllocator   *allocator);
+
+#define	G_ALLOCATOR_LIST	(1)
+#define	G_ALLOCATOR_SLIST	(2)
+#define	G_ALLOCATOR_NODE	(3)
+
 
 /* "g_mem_chunk_new" creates a new memory chunk.
  * Memory chunks are used to allocate pieces of memory which are
@@ -1642,6 +1632,7 @@ void	 g_string_sprintfa  (GString	 *string,
 
 #define g_array_append_val(a,v) g_array_append_vals(a,&v,1)
 #define g_array_prepend_val(a,v) g_array_prepend_vals(a,&v,1)
+#define g_array_insert_val(a,i,v) g_array_prepend_vals(a,i,&v,1)
 #define g_array_index(a,t,i) (((t*)a->data)[i])
 
 GArray* g_array_new	          (gboolean	    zero_terminated,
@@ -1655,6 +1646,10 @@ GArray* g_array_append_vals       (GArray	   *array,
 GArray* g_array_prepend_vals      (GArray	   *array,
 				   gconstpointer    data,
 				   guint	    len);
+GArray* g_array_insert_vals       (GArray          *array,
+				   guint            index,
+				   gconstpointer    data,
+				   guint            len);
 GArray* g_array_set_size          (GArray	   *array,
 				   guint	    length);
 GArray* g_array_remove_index	  (GArray	   *array,
@@ -2035,6 +2030,211 @@ GList*	     g_completion_complete     (GCompletion*	cmp,
 void	     g_completion_free	       (GCompletion*	cmp);
 
 
+
+
+/* Date calculations (not time for now, to be resolved). These are a
+ * mutant combination of Steffen Beyer's DateCalc routines
+ * (http://www.perl.com/CPAN/authors/id/STBEY/) and Jon Trowbridge's
+ * date routines (written for in-house software).  Written by Havoc
+ * Pennington <hp@pobox.com> 
+ */
+
+/* this enum is used to specify order of appearance in parsed date
+ *  strings
+ */
+
+typedef enum {
+  G_DATE_DAY   = 0,
+  G_DATE_MONTH = 1,
+  G_DATE_YEAR  = 2
+} GDateDMY;
+
+/* These other types specify the actual values. */
+
+typedef enum {
+  G_DATE_BAD_WEEKDAY  = 0,
+  G_DATE_MONDAY       = 1,
+  G_DATE_TUESDAY      = 2,
+  G_DATE_WEDNESDAY    = 3,
+  G_DATE_THURSDAY     = 4,
+  G_DATE_FRIDAY       = 5,
+  G_DATE_SATURDAY     = 6,
+  G_DATE_SUNDAY       = 7
+} GDateWeekday;
+
+typedef enum {
+  G_DATE_BAD_MONTH = 0,
+  G_DATE_JANUARY   = 1,
+  G_DATE_FEBRUARY  = 2,
+  G_DATE_MARCH     = 3,
+  G_DATE_APRIL     = 4,
+  G_DATE_MAY       = 5,
+  G_DATE_JUNE      = 6,
+  G_DATE_JULY      = 7,
+  G_DATE_AUGUST    = 8,
+  G_DATE_SEPTEMBER = 9,
+  G_DATE_OCTOBER   = 10,
+  G_DATE_NOVEMBER  = 11,
+  G_DATE_DECEMBER  = 12
+} GDateMonth;
+
+typedef guint16 GDateYear;
+typedef guint8  GDateDay;   /* day of the month */
+
+#define G_DATE_BAD_JULIAN 0U
+#define G_DATE_BAD_DAY    0U
+#define G_DATE_BAD_YEAR   0U
+
+typedef struct _GDate GDate;
+
+/* Note: directly manipulating structs is generally a bad idea, but
+ * in this case it's an *incredibly* bad idea, because all or part
+ * of this struct can be invalid at any given time. Use the functions,
+ * or you will get hosed, I promise.
+ */
+
+struct _GDate { 
+  guint julian_days : 32; /* julian days representation - we use a
+                           *  bitfield hoping that 64 bit platforms
+                           *  will pack this whole struct in one big
+                           *  int 
+                           */
+
+  guint julian : 1;    /* julian is valid */
+  guint dmy    : 1;    /* dmy is valid */
+
+  /* DMY representation */
+  guint day    : 6;  
+  guint month  : 4; 
+  guint year   : 16; 
+};
+
+/* _new with no args returns an invalid date, you then have to _set() stuff 
+ * to get a usable object. You can also allocate a GDate statically,
+ * then call g_date_clear() to initialize.
+ */
+
+GDate*       g_date_new                   (void);
+GDate*       g_date_new_dmy               (GDateDay     d, 
+                                           GDateMonth   m, 
+                                           GDateYear    y);
+GDate*       g_date_new_julian            (guint32      julian_day);
+void         g_date_free                  (GDate       *d);
+
+/* check g_date_valid() after doing an operation that might fail, like
+ * _parse.  Almost all g_date operations are undefined on invalid
+ * dates (the exceptions are the mutators, since you need those to
+ * return to validity).  
+ */
+
+gboolean     g_date_valid                 (GDate       *d);
+gboolean     g_date_valid_day             (GDateDay     d);
+gboolean     g_date_valid_month           (GDateMonth   m);
+gboolean     g_date_valid_year            (GDateYear    y);
+gboolean     g_date_valid_weekday         (GDateWeekday w);
+gboolean     g_date_valid_julian          (guint32      j);
+gboolean     g_date_valid_dmy             (GDateDay     d,
+                                           GDateMonth   m,
+                                           GDateYear    y);
+
+GDateWeekday g_date_weekday               (GDate       *d);
+GDateMonth   g_date_month                 (GDate       *d);
+GDateYear    g_date_year                  (GDate       *d);
+GDateDay     g_date_day                   (GDate       *d);
+guint32      g_date_julian                (GDate       *d);
+
+guint        g_date_day_of_year           (GDate       *d);
+
+/* First monday/sunday is the start of week 1; if we haven't reached
+ *  that day, return 0. These are not ISO weeks of the year; that
+ *  routine should be added.
+ */
+
+guint        g_date_monday_week_of_year   (GDate      *d); /* # weeks, starting on Monday */
+guint        g_date_sunday_week_of_year   (GDate      *d); /* # weeks, starting on Sunday */
+
+/* If you create a static date struct you need to clear it to get it
+ * in a sane state before use. You can clear a whole array at
+ * once with the ndates argument.
+ */
+
+void         g_date_clear                 (GDate       *d, 
+                                           guint        ndates);
+
+/* The parse routine is meant for dates typed in by a user, so it
+ * permits many formats but tries to catch common typos. If your data
+ * needs to be strictly validated, it is not an appropriate function.
+ */
+
+void         g_date_set_parse             (GDate       *d,
+                                           const gchar *str);
+
+void         g_date_set_time              (GDate       *d, 
+                                           GTime        time);
+
+void         g_date_set_month             (GDate       *d, 
+                                           GDateMonth   m);
+void         g_date_set_day               (GDate       *d, 
+                                           GDateDay     day);
+void         g_date_set_year              (GDate       *d,
+                                           GDateYear    y);
+void         g_date_set_dmy               (GDate       *d,
+                                           GDateDay     day,
+                                           GDateMonth   m,
+                                           GDateYear    y);
+
+void         g_date_set_julian            (GDate       *d,
+                                           guint32      j);
+
+gboolean     g_date_is_first_of_month     (GDate       *d);
+gboolean     g_date_is_last_of_month      (GDate       *d);
+
+
+/* To go forward by some number of weeks just go forward weeks*7 days */
+void         g_date_add_days              (GDate       *d, 
+                                           guint        ndays);
+void         g_date_subtract_days         (GDate       *d, 
+                                           guint        ndays);
+
+/* If you add/sub months while day > 28, the day might change */
+void         g_date_add_months            (GDate       *d,
+                                           guint        nmonths);
+void         g_date_subtract_months       (GDate       *d,
+                                           guint        nmonths);
+
+/* If it's feb 29, changing years can move you to the 28th */
+void         g_date_add_years             (GDate       *d,
+                                           guint        nyears);
+void         g_date_subtract_years        (GDate       *d,
+                                           guint        nyears);
+
+gboolean     g_date_is_leap_year          (GDateYear    year);
+guint8       g_date_days_in_month         (GDateMonth   month, 
+                                           GDateYear    year);
+guint8       g_date_monday_weeks_in_year  (GDateYear    year);
+guint8       g_date_sunday_weeks_in_year  (GDateYear    year);
+
+/* qsort-friendly (with a cast...) */
+gint         g_date_compare               (GDate       *lhs,
+                                           GDate       *rhs);
+
+/* make struct tm known without having to include time.h */
+struct tm;
+
+void         g_date_to_struct_tm          (GDate       *d,
+                                           struct tm   *tm);
+
+/* Just like strftime() except you can only use date-related formats.
+ *   Using a time format is undefined.
+ */
+gsize        g_date_strftime              (gchar       *s,
+                                           gsize        slen,
+                                           const gchar *format,
+                                           GDate       *d);
+
+
+
+
 /* GRelation: Indexed Relations.  Imagine a really simple table in a
  * database.  Relations are not ordered.  This data type is meant for
  * maintaining a N-way mapping.
@@ -2095,6 +2295,208 @@ gpointer   g_tuples_index     (GTuples	   *tuples,
 guint	   g_spaced_primes_closest (guint num);
 
 
+/* IO Channels
+ */
+
+typedef struct _GIOFuncs GIOFuncs;
+
+typedef enum {
+  G_IO_ERROR_NONE,
+  G_IO_ERROR_AGAIN,
+  G_IO_ERROR_INVAL,
+  G_IO_ERROR_UNKNOWN
+} GIOError;
+
+typedef enum {
+  G_SEEK_CUR,
+  G_SEEK_SET,
+  G_SEEK_END
+} GSeekType;
+
+typedef enum {
+  G_IO_IN  
+#ifdef POLLIN
+            = POLLIN
+#endif
+  ,G_IO_OUT  
+#ifdef POLLOUT
+            = POLLOUT
+#endif
+  ,G_IO_PRI  
+#ifdef POLLPRI
+            = POLLPRI
+#endif
+
+  ,G_IO_ERR  
+#ifdef POLLERR
+           = POLLERR
+#endif
+  ,G_IO_HUP  
+#ifdef POLLHUP
+           = POLLHUP
+#endif
+  ,G_IO_NVAL  
+#ifdef POLLNVAL
+           = POLLNVAL
+#endif
+} GIOCondition;
+
+struct _GIOChannel {
+  gpointer channel_data;
+};
+
+typedef gboolean (*GIOFunc) (GIOChannel   *source,
+			     GIOCondition  condition,
+			     gpointer      data);
+
+struct _GIOFuncs {
+  GIOError (*io_read)   (GIOChannel *channel, 
+		         gchar      *buf, 
+		         guint       count,
+			 guint      *bytes_read);
+  GIOError (*io_write)  (GIOChannel *channel, 
+		 	 gchar      *buf, 
+			 guint       count,
+			 guint      *bytes_written);
+  GIOError (*io_seek)   (GIOChannel   *channel, 
+		 	 gint        offset, 
+		  	 GSeekType   type);
+  void (*io_close)      (GIOChannel *channel);
+  guint (*io_add_watch) (GIOChannel      *channel,
+			 gint           priority,
+			 GIOCondition   condition,
+			 GIOFunc        func,
+			 gpointer       user_data,
+			 GDestroyNotify notify);
+  void (*io_free)       (GIOChannel *channel);
+};
+
+GIOChannel *g_io_channel_new    (GIOFuncs      *funcs,
+				 gpointer       channel_data);
+void      g_io_channel_ref      (GIOChannel    *channel);
+void      g_io_channel_unref    (GIOChannel    *channel);
+GIOError  g_io_channel_read     (GIOChannel    *channel, 
+			         gchar         *buf, 
+			         guint          count,
+			         guint         *bytes_read);
+GIOError  g_io_channel_write    (GIOChannel    *channel, 
+			         gchar         *buf, 
+			         guint          count,
+			         guint         *bytes_written);
+GIOError  g_io_channel_seek     (GIOChannel    *channel,
+			         gint           offset, 
+			         GSeekType      type);
+void      g_io_channel_close    (GIOChannel    *channel);
+guint     g_io_add_watch_full   (GIOChannel    *channel,
+			         gint           priority,
+			         GIOCondition   condition,
+			         GIOFunc        func,
+			         gpointer       user_data,
+			         GDestroyNotify notify);
+guint    g_io_add_watch         (GIOChannel    *channel,
+			         GIOCondition   condition,
+			         GIOFunc        func,
+			         gpointer       user_data);
+
+/* Main loop */
+
+typedef struct _GTimeVal GTimeVal;
+typedef struct _GSourceFuncs GSourceFuncs;
+
+typedef struct _GMainLoop GMainLoop; /* Opaque */
+
+struct _GTimeVal {
+  glong tv_sec;
+  glong tv_usec;
+};
+
+struct _GSourceFuncs {
+  gboolean (*prepare)  (gpointer  source_data, 
+			GTimeVal *current_time,
+			gint     *timeout);
+  gboolean (*check)    (gpointer  source_data,
+			GTimeVal *current_time);
+  gboolean (*dispatch) (gpointer  source_data, 
+			GTimeVal *current_time,
+			gpointer  user_data);
+  GDestroyNotify destroy;
+};
+
+typedef gboolean (*GSourceFunc) (gpointer        data);
+
+/* Hooks for adding to the main loop */
+
+guint g_source_add                  (gint           priority, 
+				     gboolean       can_recurse,
+				     GSourceFuncs  *funcs,
+				     gpointer       source_data, 
+				     gpointer       user_data,
+				     GDestroyNotify notify);
+void g_source_remove                (guint          tag);
+void g_source_remove_by_user_data   (gpointer       user_data);
+void g_source_remove_by_source_data (gpointer       source_data);
+
+
+void g_get_current_time (GTimeVal *result);
+
+/* Running the main loop */
+
+GMainLoop *g_main_new (void);
+void g_main_run (GMainLoop *loop);
+void g_main_quit (GMainLoop *loop);
+void g_main_destroy (GMainLoop *loop);
+
+/* Run a single iteration of the mainloop. If block is FALSE,
+ * will never block
+ */
+gboolean g_main_iteration (gboolean block);
+
+/* See if any events are pending
+ */
+gboolean g_main_pending ();
+
+/* Idles and timeouts */
+
+guint g_timeout_add_full  (gint           priority,
+			   guint          interval, 
+			   GSourceFunc    function,
+			   gpointer       data,
+			   GDestroyNotify notify);
+guint g_timeout_add       (guint          interval,
+			   GSourceFunc    function,
+			   gpointer       data);
+
+guint	   g_idle_add	   (GSourceFunc        function,
+			    gpointer	       data);
+guint	   g_idle_add_full (gint   	       priority,
+			    GSourceFunc	       function,
+			    gpointer	       data,
+			    GDestroyNotify     destroy);
+
+/* Unix-specific IO and main loop calls */
+
+typedef struct _GPollFD GPollFD;
+
+struct _GPollFD {
+   gint fd;
+   gushort events;
+   gushort revents;
+};
+
+typedef gint (*GPollFunc) (GPollFD *ufds, guint nfsd, gint timeout);
+
+void        g_main_poll_add          (gint        priority,
+				      GPollFD    *fd);
+void        g_main_poll_remove       (GPollFD    *fd);
+
+void        g_main_set_poll_func     (GPollFunc   func);
+
+
+GIOChannel *g_io_channel_unix_new    (int         fd);
+gint        g_io_channel_unix_get_fd (GIOChannel *channel);
+
+#if 0 /* old IO Channels */
+
 /* IO Channels.
  * These are used for plug-in communication in the GIMP, for instance.
  * On Unix, it's simply an encapsulated file descriptor (a pipe).
@@ -2112,8 +2514,8 @@ struct _GIOChannel
 				 * be included in the message posted
 				 * so we keep in sync.
 				 */
-  guint peer_offset;		/* in input channels where the writer's
-				 * offset is, so we don't try to read too much
+  guint need_wakeups;		/* in output channels whether the reader
+				 * needs wakeups
 				 */
 #endif
 };
@@ -2126,6 +2528,7 @@ void        g_iochannel_wakeup_peer    (GIOChannel *channel);
 #  define   g_iochannel_wakeup_peer(channel) G_STMT_START { } G_STMT_END
 #endif
 
+#endif /* old IO Channels */
 
 /* Windows emulation stubs for common unix functions
  */
