@@ -115,3 +115,13 @@ _g_module_symbol (gpointer	  handle,
   
   return p;
 }
+
+static gchar*
+_g_module_build_path (const gchar *directory,
+		      const gchar *module_name)
+{
+  if (directory)
+    return g_strconcat (directory, "/lib", module_name, ".so", NULL);
+  else
+    return g_strconcat ("lib", module_name, ".so", NULL);
+}
